/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.kernel.configuration;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;

import org.ow2.petals.jbi.messaging.routing.RouterService;
import org.ow2.petals.kernel.api.server.PetalsServer;
import org.ow2.petals.kernel.api.server.util.SystemUtil;
import org.ow2.petals.system.repository.RepositoryService;
import org.ow2.petals.tools.ws.WebServiceManager;

/**
 * This class holds the overall configuration of a PEtALS container. It stores
 * both the local topology information and the global server properties
 * information.<br>
 * The configuration service retrieves the configuration when it is started and
 * creates an instance of this class. The instance is exposed on the distributed
 * JNDI server.
 * 
 * @author alouis, rnaudin, chamerling - EBM WebSourcing
 * @author Mohamed Mokhtari
 * @since Petals 2.0
 * 
 */
public final class ContainerConfiguration implements Serializable {

    /**
     * the "unique" ID for serialization
     */
    private static final long serialVersionUID = 6543213857L;

    /**
     * Container state enumeration.
     */
    public static enum ContainerState {

        /**
         * Started state
         */
        STARTED,
        /**
         * Stopped state
         */
        STOPPED,
        /**
         * unknown state
         */
        UNKNOWN
    };

    /**
     * Registry (and maybe more) policy - related stuff.
     */
    public static enum RegistryMode {

        /**
         * Standalone mode.
         */
        STANDALONE,
        /**
         * Peer mode, used for flooding based strategy.
         */
        PEER,
        /**
         * Master mode for master / slave based strategy.
         */
        MASTER,
        /**
         * Slave mode for master / slave based strategy.
         */
        SLAVE
    };
    // ********** Global ************//
    /**
     * Subdomain name.
     */
    private String subdomainName;
    /**
     * Container name.
     */
    private String name;

    /**
     * Container description
     */
    private String description;

    /**
     * Host of the container.
     */
    private String host;

    /**
     * State of the container.
     */
    private ContainerState state = ContainerState.UNKNOWN;

    /**
     * The root directory path. This is the root path of the petals container.
     */
    private String rootDirectoryPath;

    /**
     * Repository directory path.
     */
    private String repositoryDirectoryPath;

    /**
     * The work directory where temporary things are stored
     */
    private String workDirectoryPath;

    /**
     * Exchange validation.<br>
     * If <code>true</code>, used by the Address resolver to know if a component
     * is agree to receive a exchange from a consumer.
     */
    private boolean exchangeValidation;

    /**
     * The timeout for processing the installation and deployment tasks.
     */
    private long taskTimeout;

    /**
     * Flag indicating if the JBI classLoaders must be isolated from the
     * container ClassLoader.
     */
    private boolean isolateJBIClassLoaders;

    /**
     * Flag indicating if the autoloader is activated or not.
     */
    private boolean activateAutoloader = true;

    // ************ Admin **************//

    /**
     * Login to connect to the node.
     */
    private String user;

    /**
     * Password to connect to the node.
     */
    private String password;

    /**
     * Port of the RMI adaptor for JMX.
     */
    private int jmxRMIConnectorPort;

    /**
     * Web Service Service
     */
    private int webservicePort = WebServiceManager.DEFAULT_PORT;

    private String webservicePrefix = WebServiceManager.DEFAULT_PREFIX;

    // ***** Router *****//

    /**
     * Router Quality of Service. {@see RouterService}
     */
    private String routerQOS = RouterService.DEFAULT_POLICY;

    /**
     * Router Strategy. {@see EndpointResolverModule}
     */
    private String routerStrategy = RouterService.DEFAULT_PLATFORM_STRATEGY;

    /**
     * Number of attempt to send a message 
     */
    private short routerSendAttempt = 2;

    /**
     * Delay between each send attempt 
     */
    private int routerSendDelay = 1000;

    // ***** Topology *****//

    /**
     * The topology update period, in seconds.
     */
    private long topologyUpdatePeriod = 30;


    // ***** Transporter *****//

    /**
     * Port of the TCP communication.
     */
    private int tcpPort;

    /**
     * The SSL key password
     */
    private String sslKeyPassword;

    /**
     * Number of message that can be received via TCP at the same time
     */
    private int tcpReceivers = 10;

    /**
     * Number of message that can be send via TCP at the same time, per
     * component
     */
    private int tcpSenders = 10;

    /**
     * TCP Connection timeout
     */
    private long tcpConnectionTimeout = 5000;

    /**
     * TCP send timeout
     */
    private long tcpSendTimeout = 5000;

    /**
     * The delay before running the 'sender' eviction thread
     */
    private long tcpSenderEvictorDelay = 60000;

    /**
     * The delay before an idle 'sender' is set evictable
     */
    private long tcpSenderEvictableDelay = 60000;

    // ***** Security *****//

    /**
     * The SSL Keystore file
     */
    private String sslKeystore;

    /**
     * The SSL Keystore password
     */
    private String sslKeystorePassword;

    /**
     * The SSL Truststore file
     */
    private String sslTruststore;

    /**
     * The SSL Truststore password
     */
    private String sslTruststorePassword;

    // ***** Registry *****//

    /**
     * The Registry Service
     */
    private int registryPort;

    /**
     * The registry mode.
     */
    private RegistryMode registryMode = null;

    /**
     * The registry synchro period.
     */
    private int registrySynchroPeriod = 0;

    /**
     * The registry transport timeout.
     */
    private int registryTransportTimeout = 0;

    /**
     * The registry incoming manager class name.
     */
    private String registryIncomingManagerClassName = null;

    /**
     * The registry outgoing manager class name.
     */
    private String registryOutgoingManagerClassName = null;

    /**
     * The registry message sender class name.
     */
    private String registryMessageSenderClassName = null;

    /**
     * The registry message receiver class name.
     */
    private String registryMessageReceiverClassName = null;

    /**
     * The registry repository.
     */
    private String registryRepository = null;

    /**
     * The registry database driver.
     */
    private String registryDatabaseDriver = null;

    /**
     * The registry database location.
     */
    private String registryDatabaseUrl = null;

    /**
     * The registry database user.
     */
    private String registryDatabaseUser = null;

    /**
     * The registry database password.
     */
    private String registryDatabasePassword = null;

    /**
     * The registry database dialect.
     */
    private String registryDatabaseDialect = null;
    
    /**
     * The additional configuration in the server.properties defined by the
     * user. Get all the user.* key/value pairs.
     */
    private Map<String, String> userConfiguration;

    /**
     * Default constructor
     */
    public ContainerConfiguration() {
        super();
    }

    public void init() throws ConfigurationException {
        try {
            this.setRootDirectoryPath(SystemUtil.getPetalsInstallDirectory().getCanonicalPath());
        } catch (IOException e) {
            throw new ConfigurationException("The PEtALS install directory is not valid : " + SystemUtil.getPetalsInstallDirectory().getAbsolutePath());
        }
        this.setRepositoryDirectoryPath(this.getRootDirectoryPath() + File.separator + RepositoryService.DEFAULT_REPOSITORY_PATH);

        this.setWorkDirectoryPath(this.getRootDirectoryPath() + File.separator + PetalsServer.WORK_DIRECTORY);
    }

    /**
     * @return the serialVersionUID
     */
    public static long getSerialVersionUID() {
        return serialVersionUID;
    }

    /**
     * @return the subdomainName
     */
    public String getSubdomainName() {
        return this.subdomainName;
    }

    /**
     * @param subdomainName the subdomainName to set
     */
    public void setSubdomainName(String subdomainName) {
        this.subdomainName = subdomainName;
    }

    /**
     * @return the name
     */
    public String getName() {
        return this.name;
    }

    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return the description
     */
    public String getDescription() {
        return this.description;
    }

    /**
     * @param description the description to set
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * @return the host
     */
    public String getHost() {
        return this.host;
    }

    /**
     * @param host the host to set
     */
    public void setHost(String host) {
        this.host = host;
    }

    /**
     * @return the state
     */
    public ContainerState getState() {
        return this.state;
    }

    /**
     * @param state the state to set
     */
    public void setState(ContainerState state) {
        this.state = state;
    }

    /**
     * @return the rootDirectoryPath
     */
    public String getRootDirectoryPath() {
        return this.rootDirectoryPath;
    }

    /**
     * @param rootDirectoryPath the rootDirectoryPath to set
     */
    public void setRootDirectoryPath(String rootDirectoryPath) {
        this.rootDirectoryPath = rootDirectoryPath;
    }

    /**
     * @return the repositoryDirectoryPath
     */
    public String getRepositoryDirectoryPath() {
        return this.repositoryDirectoryPath;
    }

    /**
     * @param repositoryDirectoryPath the repositoryDirectoryPath to set
     */
    public void setRepositoryDirectoryPath(String repositoryDirectoryPath) {
        this.repositoryDirectoryPath = repositoryDirectoryPath;
    }

    /**
     * @return the workDirectoryPath
     */
    public String getWorkDirectoryPath() {
        return this.workDirectoryPath;
    }

    /**
     * @param workDirectoryPath the workDirectoryPath to set
     */
    public void setWorkDirectoryPath(String workDirectoryPath) {
        this.workDirectoryPath = workDirectoryPath;
    }

    /**
     * @return the exchangeValidation
     */
    public boolean isExchangeValidation() {
        return this.exchangeValidation;
    }

    /**
     * @param exchangeValidation the exchangeValidation to set
     */
    public void setExchangeValidation(boolean exchangeValidation) {
        this.exchangeValidation = exchangeValidation;
    }

    /**
     * @return the taskTimeout
     */
    public long getTaskTimeout() {
        return this.taskTimeout;
    }

    /**
     * @param taskTimeout the taskTimeout to set
     */
    public void setTaskTimeout(long taskTimeout) {
        this.taskTimeout = taskTimeout;
    }

    /**
     * @return the isolateJBIClassLoaders
     */
    public boolean isIsolateJBIClassLoaders() {
        return this.isolateJBIClassLoaders;
    }

    /**
     * @param isolateJBIClassLoaders the isolateJBIClassLoaders to set
     */
    public void setIsolateJBIClassLoaders(boolean isolateJBIClassLoaders) {
        this.isolateJBIClassLoaders = isolateJBIClassLoaders;
    }

    /**
     * @return the activateAutoloader
     */
    public boolean isActivateAutoloader() {
        return this.activateAutoloader;
    }

    /**
     * @param activateAutoloader the activateAutoloader to set
     */
    public void setActivateAutoloader(boolean activateAutoloader) {
        this.activateAutoloader = activateAutoloader;
    }

    /**
     * @return the user
     */
    public String getUser() {
        return this.user;
    }

    /**
     * @param user the user to set
     */
    public void setUser(String user) {
        this.user = user;
    }

    /**
     * @return the password
     */
    public String getPassword() {
        return this.password;
    }

    /**
     * @param password the password to set
     */
    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * @return the jmxRMIConnectorPort
     */
    public int getJmxRMIConnectorPort() {
        return this.jmxRMIConnectorPort;
    }

    /**
     * @param jmxRMIConnectorPort the jmxRMIConnectorPort to set
     */
    public void setJmxRMIConnectorPort(int jmxRMIConnectorPort) {
        this.jmxRMIConnectorPort = jmxRMIConnectorPort;
    }

    /**
     * @return the webservicePort
     */
    public int getWebservicePort() {
        return this.webservicePort;
    }

    /**
     * @param webservicePort the webservicePort to set
     */
    public void setWebservicePort(int webservicePort) {
        this.webservicePort = webservicePort;
    }

    /**
     * @return the webservicePrefix
     */
    public String getWebservicePrefix() {
        return this.webservicePrefix;
    }

    /**
     * @param webservicePrefix the webservicePrefix to set
     */
    public void setWebservicePrefix(String webservicePrefix) {
        this.webservicePrefix = webservicePrefix;
    }

    /**
     * @return the routerQOS
     */
    public String getTransportQOS() {
        return this.routerQOS;
    }

    /**
     * @return the routerQOS
     */
    public String getRouterQOS() {
        return this.routerQOS;
    }

    /**
     * @param routerQOS the routerQOS to set
     */
    public void setTransportQOS(String routerQOS) {
        this.setRouterQOS(routerQOS);
    }

    /**
     * @return the routerStrategy
     */
    public String getRouterStrategy() {
        return this.routerStrategy;
    }

    /**
     * @param routerStrategy the routerStrategy to set
     */
    public void setRouterStrategy(String routerStrategy) {
        this.routerStrategy = routerStrategy;
    }

    /**
     * @return the routerSendAttempt
     */
    public short getRouterSendAttempt() {
        return this.routerSendAttempt;
    }

    /**
     * @param routerSendAttempt the routerSendAttempt to set
     */
    public void setRouterSendAttempt(short routerSendAttempt) {
        this.routerSendAttempt = routerSendAttempt;
    }

    /**
     * @return the routerSendDelay
     */
    public int getRouterSendDelay() {
        return this.routerSendDelay;
    }

    /**
     * @param routerSendDelay the routerSendDelay to set
     */
    public void setRouterSendDelay(int routerSendDelay) {
        this.routerSendDelay = routerSendDelay;
    }

    /**
     * @return the tcpPort
     */
    public int getTCPPort() {
        return this.tcpPort;
    }

    /**
     * @param tcpPort the tcpPort to set
     */
    public void setTCPPort(int tcpPort) {
        this.setTcpPort(tcpPort);
    }

    /**
     * @return the sslKeyPassword
     */
    public String getSSLKeyPassword() {
        return this.sslKeyPassword;
    }

    /**
     * @param sslKeyPassword the sslKeyPassword to set
     */
    public void setSSLKeyPassword(String sslKeyPassword) {
        this.setSslKeyPassword(sslKeyPassword);
    }

    /**
     * @return the tcpReceivers
     */
    public int getTCPReceivers() {
        return this.tcpReceivers;
    }

    /**
     * @param tcpReceivers the tcpReceivers to set
     */
    public void setTCPReceivers(int tcpReceivers) {
        this.setTcpReceivers(tcpReceivers);
    }

    /**
     * @return the tcpSenders
     */
    public int getTCPSenders() {
        return this.tcpSenders;
    }

    /**
     * @param tcpSenders the tcpSenders to set
     */
    public void setTCPSenders(int tcpSenders) {
        this.setTcpSenders(tcpSenders);
    }

    /**
     * @return the tcpConnectionTimeout
     */
    public long getTCPConnectionTimeout() {
        return this.tcpConnectionTimeout;
    }

    /**
     * @param tcpConnectionTimeout the tcpConnectionTimeout to set
     */
    public void setTCPConnectionTimeout(long tcpConnectionTimeout) {
        this.setTcpConnectionTimeout(tcpConnectionTimeout);
    }

    /**
     * @return the tcpSendTimeout
     */
    public long getTCPSendTimeout() {
        return this.tcpSendTimeout;
    }

    /**
     * @param tcpSendTimeout the tcpSendTimeout to set
     */
    public void setTCPSendTimeout(long tcpSendTimeout) {
        this.setTcpSendTimeout(tcpSendTimeout);
    }

    /**
     * @return the tcpSenderEvictorDelay
     */
    public long getTCPSenderEvictorDelay() {
        return this.tcpSenderEvictorDelay;
    }

    /**
     * @param tcpSenderEvictorDelay the tcpSenderEvictorDelay to set
     */
    public void setTCPSenderEvictorDelay(long tcpSenderEvictorDelay) {
        this.setTcpSenderEvictorDelay(tcpSenderEvictorDelay);
    }

    /**
     * @return the tcpSenderEvictableDelay
     */
    public long getTCPSenderEvictableDelay() {
        return this.tcpSenderEvictableDelay;
    }

    /**
     * @param tcpSenderEvictableDelay the tcpSenderEvictableDelay to set
     */
    public void setTCPSenderEvictableDelay(long tcpSenderEvictableDelay) {
        this.setTcpSenderEvictableDelay(tcpSenderEvictableDelay);
    }

    /**
     * @return the sslKeystore
     */
    public String getSSLKeystore() {
        return this.sslKeystore;
    }

    /**
     * @param sslKeystore the sslKeystore to set
     */
    public void setSSLKeystore(String sslKeystore) {
        this.setSslKeystore(sslKeystore);
    }

    /**
     * @return the sslKeystorePassword
     */
    public String getSSLKeystorePassword() {
        return this.sslKeystorePassword;
    }

    /**
     * @param sslKeystorePassword the sslKeystorePassword to set
     */
    public void setSSLKeystorePassword(String sslKeystorePassword) {
        this.setSslKeystorePassword(sslKeystorePassword);
    }

    /**
     * @return the sslTruststore
     */
    public String getSSLTruststore() {
        return this.sslTruststore;
    }

    /**
     * @param sslTruststore the sslTruststore to set
     */
    public void setSSLTruststore(String sslTruststore) {
        this.setSslTruststore(sslTruststore);
    }

    /**
     * @return the sslTruststorePassword
     */
    public String getSSLTruststorePassword() {
        return this.sslTruststorePassword;
    }

    /**
     * @param sslTruststorePassword the sslTruststorePassword to set
     */
    public void setSSLTruststorePassword(String sslTruststorePassword) {
        this.setSslTruststorePassword(sslTruststorePassword);
    }

    /**
     * @return the registryPort
     */
    public int getRegistryPort() {
        return this.registryPort;
    }

    /**
     * @param registryPort the registryPort to set
     */
    public void setRegistryPort(int registryPort) {
        this.registryPort = registryPort;
    }

    /**
     * @return the registryMode
     */
    public RegistryMode getRegistryMode() {
        return this.registryMode;
    }

    public long getTopologyUpdatePeriod() {
        return this.topologyUpdatePeriod;
    }

    /**
     * @param registryMode the registryMode to set
     */
    public void setRegistryMode(RegistryMode registryMode) {
        this.registryMode = registryMode;
    }

    public boolean isMaster() {
        return this.getRegistryMode().equals(RegistryMode.MASTER);
    }

    public boolean isSlave() {
        return this.getRegistryMode().equals(RegistryMode.SLAVE);
    }

    public boolean isInMasterSlaveMode() {
        return this.isMaster() || this.isSlave();
    }

    public boolean isInFloodingMode() {
        return this.isPeer();
    }
    
    public boolean isPeer() {
        return this.getRegistryMode().equals(RegistryMode.PEER);
    }

    public boolean isInStandaloneMode() {
        return this.isStandalone();
    }

    public boolean isStandalone() {
        return this.getRegistryMode().equals(RegistryMode.STANDALONE);
    }

    /**
     * @return the registrySynchroPeriod
     */
    public int getRegistrySynchroPeriod() {
        return this.registrySynchroPeriod;
    }

    /**
     * @param registrySynchroPeriod the registrySynchroPeriod to set
     */
    protected void setRegistrySynchroPeriod(int registrySynchroPeriod) {
        this.registrySynchroPeriod = registrySynchroPeriod;
    }

    /**
     * @return the registryTransportTimeout
     */
    public int getRegistryTransportTimeout() {
        return this.registryTransportTimeout;
    }

    /**
     * @param registryTransportTimeout the registryTransportTimeout to set
     */
    protected void setRegistryTransportTimeout(int registryTransportTimeout) {
        this.registryTransportTimeout = registryTransportTimeout;
    }

    /**
     * @return the registryIncomingManagerClassName
     */
    public String getRegistryIncomingManagerClassName() {
        return this.registryIncomingManagerClassName;
    }

    /**
     * @param registryIncomingManagerClassName the registryIncomingManagerClassName to set
     */
    protected void setRegistryIncomingManagerClassName(String registryIncomingManagerClassName) {
        this.registryIncomingManagerClassName = registryIncomingManagerClassName;
    }

    /**
     * @return the registryOutgoingManagerClassName
     */
    public String getRegistryOutgoingManagerClassName() {
        return this.registryOutgoingManagerClassName;
    }

    /**
     * @param registryOutgoingManagerClassName the registryOutgoingManagerClassName to set
     */
    protected void setRegistryOutgoingManagerClassName(String registryOutgoingManagerClassName) {
        this.registryOutgoingManagerClassName = registryOutgoingManagerClassName;
    }

    /**
     * @return the registryMessageSenderClassName
     */
    public String getRegistryMessageSenderClassName() {
        return this.registryMessageSenderClassName;
    }

    /**
     * @param registryMessageSenderClassName the registryMessageSenderClassName to set
     */
    protected void setRegistryMessageSenderClassName(String registryMessageSenderClassName) {
        this.registryMessageSenderClassName = registryMessageSenderClassName;
    }

    /**
     * @return the registryMessageReceiverClassName
     */
    public String getRegistryMessageReceiverClassName() {
        return this.registryMessageReceiverClassName;
    }

    /**
     * @param registryMessageReceiverClassName the registryMessageReceiverClassName to set
     */
    protected void setRegistryMessageReceiverClassName(String registryMessageReceiverClassName) {
        this.registryMessageReceiverClassName = registryMessageReceiverClassName;
    }

    /**
     * @return the registryRepository
     */
    public String getRegistryRepository() {
        return this.registryRepository;
    }

    /**
     * @param registryRepository the registryRepository to set
     */
    protected void setRegistryRepository(String registryRepository) {
        this.registryRepository = registryRepository;
    }

    /**
     * @return the registryDatabaseDriver
     */
    public String getRegistryDatabaseDriver() {
        return this.registryDatabaseDriver;
    }

    /**
     * @param registryDatabaseDriver the registryDatabaseDriver to set
     */
    protected void setRegistryDatabaseDriver(String registryDatabaseDriver) {
        this.registryDatabaseDriver = registryDatabaseDriver;
    }

    /**
     * @return the registryDatabaseUrl
     */
    public String getRegistryDatabaseUrl() {
        return this.registryDatabaseUrl;
    }

    /**
     * @param registryDatabaseUrl the registryDatabaseUrl to set
     */
    protected void setRegistryDatabaseUrl(String registryDatabaseUrl) {
        this.registryDatabaseUrl = registryDatabaseUrl;
    }

    /**
     * @return the registryDatabaseUser
     */
    public String getRegistryDatabaseUser() {
        return this.registryDatabaseUser;
    }

    /**
     * @param registryDatabaseUser the registryDatabaseUser to set
     */
    protected void setRegistryDatabaseUser(String registryDatabaseUser) {
        this.registryDatabaseUser = registryDatabaseUser;
    }

    /**
     * @return the registryDatabasePassword
     */
    public String getRegistryDatabasePassword() {
        return this.registryDatabasePassword;
    }

    /**
     * @param registryDatabasePassword the registryDatabasePassword to set
     */
    protected void setRegistryDatabasePassword(String registryDatabasePassword) {
        this.registryDatabasePassword = registryDatabasePassword;
    }

    /**
     * @return the registryDatabaseDialect
     */
    public String getRegistryDatabaseDialect() {
        return this.registryDatabaseDialect;
    }

    /**
     * @param registryDatabaseDialect the registryDatabaseDialect to set
     */
    protected void setRegistryDatabaseDialect(String registryDatabaseDialect) {
        this.registryDatabaseDialect = registryDatabaseDialect;
    }

    /**
     * @return the userConfiguration
     */
    public Map<String, String> getUserConfiguration() {
        return this.userConfiguration;
    }

    /**
     * @param userConfiguration the userConfiguration to set
     */
    protected void setUserConfiguration(Map<String, String> userConfiguration) {
        this.setUserConfiguration(userConfiguration);
    }

    /**
     * @param routerQOS the routerQOS to set
     */
    public void setRouterQOS(String routerQOS) {
        this.routerQOS = routerQOS;
    }

    /**
     * @param tcpPort the tcpPort to set
     */
    public void setTcpPort(int tcpPort) {
        this.tcpPort = tcpPort;
    }

    /**
     * @param sslKeyPassword the sslKeyPassword to set
     */
    public void setSslKeyPassword(String sslKeyPassword) {
        this.sslKeyPassword = sslKeyPassword;
    }

    /**
     * @param tcpReceivers the tcpReceivers to set
     */
    public void setTcpReceivers(int tcpReceivers) {
        this.tcpReceivers = tcpReceivers;
    }

    /**
     * @param tcpSenders the tcpSenders to set
     */
    public void setTcpSenders(int tcpSenders) {
        this.tcpSenders = tcpSenders;
    }

    /**
     * @param tcpConnectionTimeout the tcpConnectionTimeout to set
     */
    public void setTcpConnectionTimeout(long tcpConnectionTimeout) {
        this.tcpConnectionTimeout = tcpConnectionTimeout;
    }

    /**
     * @param tcpSendTimeout the tcpSendTimeout to set
     */
    public void setTcpSendTimeout(long tcpSendTimeout) {
        this.tcpSendTimeout = tcpSendTimeout;
    }

    /**
     * @param tcpSenderEvictorDelay the tcpSenderEvictorDelay to set
     */
    public void setTcpSenderEvictorDelay(long tcpSenderEvictorDelay) {
        this.tcpSenderEvictorDelay = tcpSenderEvictorDelay;
    }

    /**
     * @param tcpSenderEvictableDelay the tcpSenderEvictableDelay to set
     */
    public void setTcpSenderEvictableDelay(long tcpSenderEvictableDelay) {
        this.tcpSenderEvictableDelay = tcpSenderEvictableDelay;
    }

    /**
     * @param sslKeystore the sslKeystore to set
     */
    public void setSslKeystore(String sslKeystore) {
        this.sslKeystore = sslKeystore;
    }

    /**
     * @param sslKeystorePassword the sslKeystorePassword to set
     */
    public void setSslKeystorePassword(String sslKeystorePassword) {
        this.sslKeystorePassword = sslKeystorePassword;
    }

    /**
     * @param sslTruststore the sslTruststore to set
     */
    public void setSslTruststore(String sslTruststore) {
        this.sslTruststore = sslTruststore;
    }

    /**
     * @param sslTruststorePassword the sslTruststorePassword to set
     */
    public void setSslTruststorePassword(String sslTruststorePassword) {
        this.sslTruststorePassword = sslTruststorePassword;
    }

    public void setTopologyUpdatePeriod(long topologyUpdatePeriod) {
        this.topologyUpdatePeriod = topologyUpdatePeriod;
    }


    @Override
    public String toString() {
        StringBuilder toStringBuilder = new StringBuilder();
        toStringBuilder.append("Container ").append(this.getName()).append(" on ").append(this.getHost()).append(" in sub-domain ").append(this.getSubdomainName());
        if (this.getDescription() != null) {
            toStringBuilder.append("\n\t\t\tDescription :\n\t\t\t  ").append(this.getDescription());
        }
        toStringBuilder.append("\n\t\t\tConfiguration :\n\t\t\t  - InstallDirectory:").append(
                this.getRootDirectoryPath());
        toStringBuilder.append("\n\t\t\t  - RepositoryDirectory:").append(
                this.getRepositoryDirectoryPath());
        toStringBuilder.append("\n\t\t\t  - ExchangeValidation:").append(this.isExchangeValidation());
        toStringBuilder.append("\n\t\t\t  - TransportQOS:").append(this.getTransportQOS());
        toStringBuilder.append("\n\t\t\t  - RoutingStrategy:").append(this.getRouterStrategy());
        toStringBuilder.append("\n\t\t\t  - TaskTimeout:").append(this.getTaskTimeout());
        toStringBuilder.append("\n\t\t\t  - IsolateClassLoaders:").append(
                this.isIsolateJBIClassLoaders());
        toStringBuilder.append("\n\t\t\t  - ActivateAutoloader:").append(this.isActivateAutoloader());
        toStringBuilder.append("\n\t\t\t  - User:").append((this.getUser() != null) ? this.getUser() : "none");
        if (this.getPassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getPassword().length());
            while (hiddenPassword.length() < this.getPassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - Password:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - Password:none");
        }
        toStringBuilder.append("\n\t\t\t  - JMXRMIConnectionPort:").append(this.getJmxRMIConnectorPort());
        toStringBuilder.append("\n\t\t\t  - TCPPort:").append(
                (this.getTCPPort() != 0) ? this.getTCPPort() : "none");
        if (this.getSSLKeyPassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLKeyPassword().length());
            while (hiddenPassword.length() < this.getSSLKeyPassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - sslKeyPassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - sslKeyPassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - Keystore:").append(
                (this.getSSLKeystore() != null) ? this.getSSLKeystore() : "none");
        if (this.getSSLKeystorePassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLKeystorePassword().length());
            while (hiddenPassword.length() < this.getSSLKeystorePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - KeystorePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - KeystorePassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - Truststore:").append(
                (this.getSSLTruststore() != null) ? this.getSSLTruststore() : "none");
        if (this.getSSLTruststorePassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getSSLTruststorePassword().length());
            while (hiddenPassword.length() < this.getSSLTruststorePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - TruststorePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - TruststorePassword:none");
        }
        if (this.getTCPPort() != 0) {
            toStringBuilder.append("\n\t\t\t  - TCPReceivers:").append(this.getTCPReceivers());
            toStringBuilder.append("\n\t\t\t  - TCPSenders:").append(this.getTCPSenders());
            toStringBuilder.append("\n\t\t\t  - TCPConnectionTimeout:").append(
                    this.getTCPConnectionTimeout());
            toStringBuilder.append("\n\t\t\t  - TCPSendTimeout:").append(this.getTCPSendTimeout());
            toStringBuilder.append("\n\t\t\t  - TCPSenderEvictorDelay:").append(
                    this.getTCPSenderEvictorDelay());
            toStringBuilder.append("\n\t\t\t  - TCPSenderEvictableDelay:").append(
                    this.getTCPSenderEvictableDelay());
        }
        toStringBuilder.append("\n\t\t\t  - State:").append(this.getState().name());
        toStringBuilder.append("\n\t\t\t  - RegistryPort:").append(this.getRegistryPort());
        toStringBuilder.append("\n\t\t\t  - RegistryMode:").append(this.getRegistryMode());
        toStringBuilder.append("\n\t\t\t  - RegistrySynchroPeriod:").append(this.getRegistrySynchroPeriod());
        toStringBuilder.append("\n\t\t\t  - RegistryTransportTimeout:").append(this.getRegistryTransportTimeout());
        toStringBuilder.append("\n\t\t\t  - RegistryIncomingManagerClassName:").append(this.getRegistryIncomingManagerClassName());
        toStringBuilder.append("\n\t\t\t  - RegistryOutgoingManagerClassName:").append(this.getRegistryOutgoingManagerClassName());
        toStringBuilder.append("\n\t\t\t  - RegistryMessageSenderClassName:").append(this.getRegistryMessageSenderClassName());
        toStringBuilder.append("\n\t\t\t  - RegistryMessageReceiverClassName:").append(this.getRegistryMessageReceiverClassName());
        toStringBuilder.append("\n\t\t\t  - RegistryRepository:").append(this.getRegistryRepository());
        toStringBuilder.append("\n\t\t\t  - RegistryDatabaseDriver:").append(this.getRegistryDatabaseDriver());
        toStringBuilder.append("\n\t\t\t  - RegistryDatabaseUrl:").append(this.getRegistryDatabaseUrl());
        toStringBuilder.append("\n\t\t\t  - RegistryDatabaseUser:").append(this.getRegistryDatabaseUser());
        if (this.getRegistryDatabasePassword() != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.getRegistryDatabasePassword().length());
            while (hiddenPassword.length() < this.getRegistryDatabasePassword().length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - RegistryDatabasePassword:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - RegistryDatabasePassword:none");
        }
        toStringBuilder.append("\n\t\t\t  - RegistryDatabaseDialect:").append(this.getRegistryDatabaseDialect());
        toStringBuilder.append("\n\t\t\t  - WebServicePort:").append(this.getWebservicePort());
        toStringBuilder.append("\n\t\t\t  - WebServicePrefix:").append(this.getWebservicePrefix());

        if (this.getUserConfiguration() != null) {
            StringBuffer userBuffer = new StringBuffer();
            Iterator<String> iter = this.getUserConfiguration().keySet().iterator();
            while (iter.hasNext()) {
                String key = iter.next();
                userBuffer.append("\n\t\t\t\t").append(key).append('=').append(
                        this.getUserConfiguration().get(key));
            }
            toStringBuilder.append("\n\t\t\t  - User properties:").append(userBuffer.toString());
        }

        toStringBuilder.append("\n\t\t\t  - Topology update period:").append(this.getTopologyUpdatePeriod());

        return toStringBuilder.toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (this.isActivateAutoloader() ? 1231 : 1237);
        result = prime * result + ((this.getDescription() == null) ? 0 : this.getDescription().hashCode());
        result = prime * result + this.getTCPPort();
        result = prime * result + this.getTCPReceivers();
        result = prime * result + this.getTCPSenders();
        result = prime * result + (int) (this.getTCPConnectionTimeout() ^ (this.getTCPConnectionTimeout() >>> 32));
        result = prime * result + (int) (this.getTCPSendTimeout() ^ (this.getTCPSendTimeout() >>> 32));
        result = prime * result + (int) (this.getTCPSenderEvictorDelay() ^ (this.getTCPSenderEvictorDelay() >>> 32));
        result = prime * result + (int) (this.getTCPSenderEvictableDelay() ^ (this.getTCPSenderEvictableDelay() >>> 32));
        result = prime * result + (this.isExchangeValidation() ? 1231 : 1237);
        result = prime * result + ((this.getHost() == null) ? 0 : this.getHost().hashCode());
        result = prime * result + ((this.getRootDirectoryPath() == null) ? 0 : this.getRootDirectoryPath().hashCode());
        result = prime * result + (this.isIsolateJBIClassLoaders() ? 1231 : 1237);
        result = prime * result + this.getJmxRMIConnectorPort();
        result = prime * result + ((this.getSSLKeyPassword() == null) ? 0 : this.getSSLKeyPassword().hashCode());
        result = prime * result + ((this.getSSLKeystore() == null) ? 0 : this.getSSLKeystore().hashCode());
        result = prime * result + ((this.getSSLKeystorePassword() == null) ? 0 : this.getSSLKeystorePassword().hashCode());
        result = prime * result + ((this.getName() == null) ? 0 : this.getName().hashCode());
        result = prime * result + ((this.getPassword() == null) ? 0 : this.getPassword().hashCode());
        result = prime * result + ((this.getRepositoryDirectoryPath() == null) ? 0 : this.getRepositoryDirectoryPath().hashCode());
        result = prime * result + ((this.getRouterStrategy() == null) ? 0 : this.getRouterStrategy().hashCode());
        result = prime * result + ((this.getState() == null) ? 0 : this.getState().hashCode());
        result = prime * result + ((this.getSubdomainName() == null) ? 0 : this.getSubdomainName().hashCode());
        result = prime * result + (int) (this.getTaskTimeout() ^ (this.getTaskTimeout() >>> 32));
        result = prime * result + ((this.getTransportQOS() == null) ? 0 : this.getTransportQOS().hashCode());
        result = prime * result + ((this.getSSLTruststore() == null) ? 0 : this.getSSLTruststore().hashCode());
        result = prime * result + ((this.getSSLTruststorePassword() == null) ? 0 : this.getSSLTruststorePassword().hashCode());
        result = prime * result + ((this.getUser() == null) ? 0 : this.getUser().hashCode());
        result = prime * result + this.getWebservicePort();
        result = prime * result + ((this.getWebservicePrefix() == null) ? 0 : this.getWebservicePrefix().hashCode());
        result = prime * result + ((this.getUserConfiguration() == null) ? 0 : this.getUserConfiguration().hashCode());
        result = prime * result + this.getRegistryPort();
        result = prime * result + ((this.getRegistryMode() == null) ? 0 : this.getRegistryMode().hashCode());
        result = prime * result + this.getRegistrySynchroPeriod();
        result = prime * result + this.getRegistryTransportTimeout();
        result = prime * result + ((this.getRegistryIncomingManagerClassName() == null) ? 0 : this.getRegistryIncomingManagerClassName().hashCode());
        result = prime * result + ((this.getRegistryOutgoingManagerClassName() == null) ? 0 : this.getRegistryOutgoingManagerClassName().hashCode());
        result = prime * result + ((this.getRegistryMessageSenderClassName() == null) ? 0 : this.getRegistryMessageSenderClassName().hashCode());
        result = prime * result + ((this.getRegistryMessageReceiverClassName() == null) ? 0 : this.getRegistryMessageReceiverClassName().hashCode());
        result = prime * result + ((this.getRegistryRepository() == null) ? 0 : this.getRegistryRepository().hashCode());
        result = prime * result + ((this.getRegistryDatabaseDriver() == null) ? 0 : this.getRegistryDatabaseDriver().hashCode());
        result = prime * result + ((this.getRegistryDatabaseUrl() == null) ? 0 : this.getRegistryDatabaseUrl().hashCode());
        result = prime * result + ((this.getRegistryDatabaseUser() == null) ? 0 : this.getRegistryDatabaseUser().hashCode());
        result = prime * result + ((this.getRegistryDatabasePassword() == null) ? 0 : this.getRegistryDatabasePassword().hashCode());
        result = prime * result + ((this.getRegistryDatabaseDialect() == null) ? 0 : this.getRegistryDatabaseDialect().hashCode());
        result = prime * result + ((int) this.getTopologyUpdatePeriod());

        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        boolean result = false;
        if (obj instanceof ContainerConfiguration) {
            ContainerConfiguration otherContainerConfiguration = (ContainerConfiguration) obj;
            result = (otherContainerConfiguration.getName().equals(this.getName())
                    && otherContainerConfiguration.getHost().equals(this.getHost())
                    && (otherContainerConfiguration.isExchangeValidation() == this.isExchangeValidation())
                    && otherContainerConfiguration.getTransportQOS().equals(this.getTransportQOS())
                    && otherContainerConfiguration.getRouterStrategy().equals(this.getRouterStrategy())
                    && (otherContainerConfiguration.getTaskTimeout() == this.getTaskTimeout())
                    && (otherContainerConfiguration.isIsolateJBIClassLoaders() == this.isIsolateJBIClassLoaders())
                    && (otherContainerConfiguration.isActivateAutoloader() == this.isActivateAutoloader())
                    && otherContainerConfiguration.getUser().equals(this.getUser())
                    && otherContainerConfiguration.getPassword().equals(this.getPassword())
                    && (otherContainerConfiguration.getJmxRMIConnectorPort() == this.getJmxRMIConnectorPort())
                    && (otherContainerConfiguration.getTCPPort() == this.getTCPPort())
                    && (otherContainerConfiguration.getRegistryMode() == this.getRegistryMode())
                    && (otherContainerConfiguration.getTCPReceivers() == this.getTCPReceivers())
                    && (otherContainerConfiguration.getTCPSenders() == this.getTCPSenders())
                    && (otherContainerConfiguration.getTCPConnectionTimeout() == this.getTCPConnectionTimeout())
                    && (otherContainerConfiguration.getTCPSendTimeout() == this.getTCPSendTimeout())
                    && (otherContainerConfiguration.getTCPSenderEvictorDelay() == this.getTCPSenderEvictorDelay())
                    && (otherContainerConfiguration.getTCPSenderEvictableDelay() == this.getTCPSenderEvictableDelay())
                    && otherContainerConfiguration.getSSLKeyPassword().equals(this.getSSLKeyPassword())
                    && otherContainerConfiguration.getSSLKeystore().equals(this.getSSLKeystore())
                    && otherContainerConfiguration.getSSLKeystorePassword().equals(this.getSSLKeystorePassword())
                    && otherContainerConfiguration.getSSLTruststore().equals(this.getSSLTruststore())
                    && otherContainerConfiguration.getSSLTruststorePassword().equals(this.getSSLTruststorePassword())
                    && (otherContainerConfiguration.getWebservicePort() == (this.getWebservicePort()))
                    && otherContainerConfiguration.getWebservicePrefix().equals(this.getWebservicePrefix())
                    && (otherContainerConfiguration.getRegistryPort() == this.getRegistryPort())
                    && otherContainerConfiguration.getRegistryMode().equals(this.getRegistryMode())
                    && (otherContainerConfiguration.getRegistrySynchroPeriod() == this.getRegistrySynchroPeriod())
                    && (otherContainerConfiguration.getRegistryTransportTimeout() == this.getRegistryTransportTimeout())
                    && otherContainerConfiguration.getRegistryIncomingManagerClassName().equals(this.getRegistryIncomingManagerClassName())
                    && otherContainerConfiguration.getRegistryOutgoingManagerClassName().equals(this.getRegistryOutgoingManagerClassName())
                    && otherContainerConfiguration.getRegistryMessageSenderClassName().equals(this.getRegistryMessageSenderClassName())
                    && otherContainerConfiguration.getRegistryMessageReceiverClassName().equals(this.getRegistryMessageReceiverClassName())
                    && otherContainerConfiguration.getRegistryRepository().equals(this.getRegistryRepository())
                    && otherContainerConfiguration.getRegistryDatabaseDriver().equals(this.getRegistryDatabaseDriver())
                    && otherContainerConfiguration.getRegistryDatabaseUrl().equals(this.getRegistryDatabaseUrl())
                    && otherContainerConfiguration.getRegistryDatabaseUser().equals(this.getRegistryDatabaseUser())
                    && otherContainerConfiguration.getRegistryDatabasePassword().equals(this.getRegistryDatabasePassword())
                    && otherContainerConfiguration.getRegistryDatabaseDialect().equals(this.getRegistryDatabaseDialect())
                    && ((otherContainerConfiguration.getUserConfiguration() == this.getUserConfiguration()) || ((otherContainerConfiguration.getUserConfiguration() != null)
                    && otherContainerConfiguration.getUserConfiguration().equals(this.getUserConfiguration()))))
                    && (otherContainerConfiguration.getTopologyUpdatePeriod() == this.getTopologyUpdatePeriod());
        }
        return result;
    }
}
