/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 eBM Websourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: RepositoryServiceImpl.java 13311 2009-11-25 14:42:47Z rnaudin $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.system.repository;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.ZipUtil;

/**
 * Platform Component Repository service implementation.
 * 
 * @version $Rev: 13311 $ $Date: 2009-08-03 10:08:35 +0200 (lun., 03 août 2009)
 *          $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author alouis - EBM Websourcing
 * @author ofabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.system.repository.RepositoryService.class))
public class RepositoryServiceImpl implements RepositoryService {

    /**
     * Entity types (components, shared libs or service assemblies)
     */
    public static enum EntityType {
        COMPONENT_TYPE, SA_TYPE, SL_TYPE
    }

    /**
     * Path to components directory into petals repository
     */
    private static final String COMPONENTS_PATH = "components";

    /**
     * The date pattern
     */
    private static final String DATE_PATTERN = "MM-dd-yyyy-hhmmssSSS";

    /**
     * Path to component install directory
     */
    private static final String INSTALL_PATH = "install";

    /**
     * Path to lost+found directory into petals repository
     */
    private static final String LOSTPLUSFOUND_PATH = "lost+found";

    /**
     * Path to components directory into petals repository
     */
    private static final String SERVICEASSEMBLIES_PATH = "service-assemblies";

    /**
     * Path to components directory into petals repository
     */
    private static final String SHAREDLIBRARIES_PATH = "shared-libraries";

    /**
     * Path to component working directory
     */
    private static final String WORK_PATH = "work";

    /**
     * logger wrapper
     */
    protected LoggingUtil log;;

    /**
     * Monolog Logger instance.
     */
    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * The components directory path.
     */
    private File componentsDirectory;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * DateFormat used for repository folders
     */
    private SimpleDateFormat folderDateFormat;

    /**
     * The lost+found directory path.
     */
    private File lostFoundDirectory;

    /**
     * The complete repository directory path.
     */
    private File repositoryDirectory;

    /**
     * The service assemblies directory path.
     */
    private File serviceAssembliesDirectory;

    /**
     * The shared libraries directory path.
     */
    private File sharedLibrariesDirectory;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#addComponent(java.
     * lang.String, java.io.File)
     */
    public File addComponent(String componentId, File tempFile) throws IOException {
        return this.addPackage(componentId, tempFile, EntityType.COMPONENT_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#addServiceAssembly
     * (java.lang.String, java.io.File)
     */
    public File addServiceAssembly(String serviceAssemblyId, File tempFile) throws IOException {
        return this.addPackage(serviceAssemblyId, tempFile, EntityType.SA_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#addSharedLibrary(java
     * .lang.String, java.io.File)
     */
    public File addSharedLibrary(String sharedLibraryId, File tempFile) throws IOException {
        return this.addPackage(sharedLibraryId, tempFile, EntityType.SL_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * explodeSUIntoSAInstallDirectory(java.lang.String, java.net.URL,
     * java.lang.String)
     */
    public File explodeSUIntoSAInstallDirectory(String serviceUnitId, URL suZipLocation,
            String serviceAssemblyId) throws IOException {
        File installationRoot = this.createSUInstallDirectory(serviceUnitId, serviceAssemblyId);
        try {
            File zipArchive = new File(suZipLocation.getPath());
            ZipUtil.unzipArchive(zipArchive, installationRoot);
        } catch (IOException e) {
            try {
                FileUtils.forceDelete(installationRoot);
            } catch (IOException e1) {
                // Nothing to do, just log
                this.log.warning("Failed to clean a failed installation", e);
            }
            throw e;
        }
        return installationRoot;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getComponentDirectory
     * (java.lang.String)
     */
    public File getComponentDirectory(String componentId) throws IOException {
        return this.getEntityRoot(componentId, EntityType.COMPONENT_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getComponentInstallDirectory(java.lang.String)
     */
    public File getComponentInstallDirectory(String componentId) throws IOException {
        return new File(this.getComponentDirectory(componentId), INSTALL_PATH);

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getComponentsDirectory
     * ()
     */
    public File getComponentsDirectory() {
        return this.componentsDirectory;

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getComponentWorkDirectory
     * (java.lang.String)
     */
    public File getComponentWorkDirectory(String componentId) throws IOException {
        return new File(this.getComponentDirectory(componentId), WORK_PATH);

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getLostPlusFoundDirectory
     * ()
     */
    public File getLostPlusFoundDirectory() {
        return this.lostFoundDirectory;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getRepositoryDirectory
     * ()
     */
    public File getRepositoryDirectory() {
        return this.repositoryDirectory;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getServiceAssembliesDirectory()
     */
    public File getServiceAssembliesDirectory() {
        return this.serviceAssembliesDirectory;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getServiceAssemblyDirectory(java.lang.String)
     */
    public File getServiceAssemblyDirectory(String serviceAssemblyId) throws IOException {
        return this.getEntityRoot(serviceAssemblyId, EntityType.SA_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getServiceAssemblyInstallDirectory(java.lang.String)
     */
    public File getServiceAssemblyInstallDirectory(String serviceAssemblyId) throws IOException {
        return new File(this.getServiceAssemblyDirectory(serviceAssemblyId), INSTALL_PATH);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getServiceAssemblyWorkDirectory(java.lang.String)
     */
    public File getServiceAssemblyWorkDirectory(String serviceAssemblyId) throws IOException {
        return new File(this.getServiceAssemblyDirectory(serviceAssemblyId), WORK_PATH);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getSharedLibrariesDirectory()
     */
    public File getSharedLibrariesDirectory() {
        return this.sharedLibrariesDirectory;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#getSharedLibraryDirectory
     * (java.lang.String)
     */
    public File getSharedLibraryDirectory(String sharedLibraryId) throws IOException {
        return this.getEntityRoot(sharedLibraryId, EntityType.SL_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getSharedLibraryInstallDirectory(java.lang.String)
     */
    public File getSharedLibraryInstallDirectory(String sharedLibraryId) throws IOException {
        String name = INSTALL_PATH;
        return new File(this.getSharedLibraryDirectory(sharedLibraryId), name);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.system.repository.RepositoryService#
     * getSharedLibraryWorkDirectory(java.lang.String)
     */
    public File getSharedLibraryWorkDirectory(String sharedLibraryId) throws IOException {
        return new File(this.getSharedLibraryDirectory(sharedLibraryId), WORK_PATH);

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#removeComponent(java
     * .lang.String)
     */
    public void removeComponent(String componentId) throws IOException {
        this.removePackage(componentId, EntityType.COMPONENT_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#removeServiceAssembly
     * (java.lang.String)
     */
    public void removeServiceAssembly(String serviceAssemblyId) throws IOException {
        this.removePackage(serviceAssemblyId, EntityType.SA_TYPE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.system.repository.RepositoryService#removeSharedLibrary
     * (java.lang.String)
     */
    public void removeSharedLibrary(String sharedLibraryId) throws IOException {
        this.removePackage(sharedLibraryId, EntityType.SL_TYPE);
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IOException {
        this.log = new LoggingUtil(this.logger);
        this.log.start();

        this.folderDateFormat = new SimpleDateFormat(DATE_PATTERN);
        this.containerConfiguration = this.configurationService.getContainerConfiguration();

        this.repositoryDirectory = new File(this.containerConfiguration
                .getRepositoryDirectoryPath());

        try {
            if (!this.repositoryDirectory.exists() && !this.repositoryDirectory.mkdir()) {
                throw new IOException("Directory '" + this.repositoryDirectory.getAbsolutePath()
                        + "' can not be create");
            }

            this.lostFoundDirectory = new File(this.repositoryDirectory, LOSTPLUSFOUND_PATH);
            if (!this.lostFoundDirectory.exists()) {
                this.lostFoundDirectory.mkdir();
            }

            this.componentsDirectory = new File(this.repositoryDirectory, COMPONENTS_PATH);
            if (!this.componentsDirectory.exists()) {
                this.componentsDirectory.mkdir();
            }

            this.serviceAssembliesDirectory = new File(this.repositoryDirectory,
                    SERVICEASSEMBLIES_PATH);
            if (!this.serviceAssembliesDirectory.exists()) {
                this.serviceAssembliesDirectory.mkdir();
            }

            this.sharedLibrariesDirectory = new File(this.repositoryDirectory, SHAREDLIBRARIES_PATH);
            if (!this.sharedLibrariesDirectory.exists()) {
                this.sharedLibrariesDirectory.mkdir();
            }
        } catch (SecurityException e) {
            this.log.error(e);
            throw e;
        }

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

    /**
     * Copy an entity package into a PEtALS repository
     * 
     * @param entityId
     *            {@link String} unique entity id that identifies an unique
     *            entity directory into the petals repository.
     * @param tempFile
     *            {@link File} the entity package file to add into the
     *            repository.
     * @param entityType
     *            the entity type: component, shared lib or service assembly
     * @return installation root {@link File}
     * @throws PetalsException
     *             if entity installation or work root can't be created or if
     *             entity files can't be copied in the installation root
     */
    private File addPackage(String entityId, File tempFile, EntityType entityType)
            throws IOException {

        File rootDirectory = this.createEntityDirectory(entityId, entityType);
        File installationRoot = this.createEntityInstallDirectory(rootDirectory);
        File workRoot = this.createEntityWorkDirectory(rootDirectory);

        try {
            FileUtils.copyDirectory(tempFile, installationRoot);
            FileUtils.forceDelete(tempFile);
        } catch (IOException e) {
            try {
                FileUtils.forceDelete(rootDirectory);
                FileUtils.forceDelete(workRoot);
            } catch (IOException e1) {
                // Nothing to do, just log
                this.log.warning("Residual incomplete component directory");
            }
            throw e;
        }

        return installationRoot;
    }

    private File createComponentDirectory(String componentId) {
        String name = componentId + "-"
                + this.folderDateFormat.format(new Date(System.currentTimeMillis()));
        File componentDirectory = new File(this.componentsDirectory, name);
        componentDirectory.mkdir();

        return componentDirectory;
    }

    private File createEntityDirectory(String entityId, EntityType entityType) throws IOException {
        File rootFile = null;

        switch (entityType) {
            case COMPONENT_TYPE:
                rootFile = this.createComponentDirectory(entityId);
                break;
            case SA_TYPE:
                rootFile = this.createServiceAssemblyDirectory(entityId);
                break;
            case SL_TYPE:
                rootFile = this.createSharedLibraryDirectory(entityId);
                break;
        }
        return rootFile;
    }

    private File createEntityInstallDirectory(File rootDirectory) {
        File installDirectory = new File(rootDirectory, INSTALL_PATH);
        installDirectory.mkdir();

        return installDirectory;
    }

    private File createEntityWorkDirectory(File rootDirectory) {
        File workDirectory = new File(rootDirectory, WORK_PATH);
        workDirectory.mkdir();

        return workDirectory;
    }

    /**
     * Create the shared library root directory.
     * 
     * @param sharedLibraryId
     * @return
     */
    private File createServiceAssemblyDirectory(String serviceAssemblyId) {
        String name = serviceAssemblyId + "-"
                + this.folderDateFormat.format(new Date(System.currentTimeMillis()));
        File serviceAssemblyDirectory = new File(this.serviceAssembliesDirectory, name);
        serviceAssemblyDirectory.mkdir();

        return serviceAssemblyDirectory;
    }

    /**
     * Create the shared library root directory.
     * 
     * @param sharedLibraryId
     * @return
     */
    private File createSharedLibraryDirectory(String sharedLibraryId) {
        String name = sharedLibraryId + "-"
                + this.folderDateFormat.format(new Date(System.currentTimeMillis()));
        File sharedLibraryDirectory = new File(this.sharedLibrariesDirectory, name);
        sharedLibraryDirectory.mkdir();

        return sharedLibraryDirectory;
    }

    /**
     * Create the SU install root into the SA install root
     * 
     * @param suId
     *            the service unit unique identifier
     * @param saId
     *            the service assembly unique identifier
     * @return the SU install root File
     * @throws PetalsException
     *             if SU install root can't be created
     */
    private File createSUInstallDirectory(String serviceUnitId, String serviceAssemblyId)
            throws IOException {
        File installationRoot = new File(
                this.getServiceAssemblyInstallDirectory(serviceAssemblyId), serviceUnitId);

        if (!installationRoot.exists()) {
            installationRoot.mkdirs();
        }

        return installationRoot;
    }

    private File getEntityRoot(String entityId, EntityType entityType) throws IOException {
        File entitiesDirectory = null;

        switch (entityType) {
            case COMPONENT_TYPE:
                entitiesDirectory = this.getComponentsDirectory();
                break;
            case SA_TYPE:
                entitiesDirectory = this.getServiceAssembliesDirectory();
                break;
            case SL_TYPE:
                entitiesDirectory = this.getSharedLibrariesDirectory();
                break;
        }

        String[] entityFilePaths = entitiesDirectory.list(new RepositoryFileFilter(entityId));
        
        //FIXME: Workaround due to the effect to the bug 314239
        String entityFile = "";
        for (String _entityFile : entityFilePaths) {
            if (entityFile.compareTo(_entityFile) < 0) {
                entityFile =_entityFile;
            }
        }
//        if (entityFilePaths.length > 1) {
//            throw new IOException("The PEtALS repository is corrupted. Please clean it!");
//        } else if (entityFilePaths.length == 0) {
//            return null;
//        }

        return new File(entitiesDirectory, entityFilePaths[0]);
    }

    private void removePackage(String entityId, EntityType entityType) throws IOException {
        File entityRootFile = null;
        switch (entityType) {
            case COMPONENT_TYPE:
                entityRootFile = this.getComponentDirectory(entityId);
                break;
            case SA_TYPE:
                entityRootFile = this.getServiceAssemblyDirectory(entityId);
                break;
            case SL_TYPE:
                entityRootFile = this.getSharedLibraryDirectory(entityId);
                break;
        }

        if (entityRootFile != null) {
            FileUtils.deleteDirectory(entityRootFile);
        }
    }

}
